/*------------------------------------------------------------------------------
--  This file is a part of the SocExplorer Software
--  Copyright (C) 2014, Plasma Physics Laboratory - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-------------------------------------------------------------------------------*/
/*--                  Author : Alexis Jeandet
--                     Mail : alexis.jeandet@lpp.polytechnique.fr
----------------------------------------------------------------------------*/
#include "memchecker.h"
#include <socexplorerengine.h>

memchecker::memchecker(socexplorerplugin* plugin)
{
  this->plugin = plugin;
}

QString memchecker::checkmemory(unsigned int address, unsigned int size, bool *success)
{
  *success = true;
  QString repport;
  unsigned int* dataLocal = (unsigned int*)malloc(size);
  unsigned int* dataOnBoard = (unsigned int*)malloc(size);
  for(int i=0;(unsigned int)i<(size>>2);i++)
    {
      dataLocal[i]= (0xFFFF&rand())+(rand()<<16);
    }
  plugin->Write(dataLocal,size>>2,address);
  plugin->Read(dataOnBoard,size>>2,address);
  for(int i=0;(unsigned int)i<(size>>2);i++)
    {
      if(dataLocal[i]!=dataOnBoard[i])
        *success=false;
    }
  free(dataLocal);
  free(dataOnBoard);
  return repport;
}

QString memchecker::checkdatabits(unsigned int address,unsigned int size,bool* success)
{
  *success = true;
  QString repport;
  return repport;
}


unsigned int p_pow2(unsigned int v)
{
  static const char LogTable256[256] =
  {
  #define LT(n) n, n, n, n, n, n, n, n, n, n, n, n, n, n, n, n
      -1, 0, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3,
      LT(4), LT(5), LT(5), LT(6), LT(6), LT(6), LT(6),
      LT(7), LT(7), LT(7), LT(7), LT(7), LT(7), LT(7), LT(7)
  };

  unsigned int r;     // r will be lg(v)
  register unsigned int t, tt; // temporaries

  if (tt = v >> 16)
  {
    r = (t = tt >> 8) ? 24 + LogTable256[t] : 16 + LogTable256[tt];
  }
  else
  {
    r = (t = v >> 8) ? 8 + LogTable256[t] : LogTable256[v];
  }
  return r;
}


unsigned int memchecker::measureMemSize(unsigned int address, unsigned int maxSize)
{
    return   SocExplorerEngine::memMeasureSize(this->plugin,address,maxSize);
}



